Returns a data.table, data.frame, or sf object of unprojected (longlat) points.
Arguments
nobs
number of observations
xmin
minimum longitude
xmax
maximum longitude
ymin
minimum latitude
ymax
maximum latitude
output_type
type of output object. Either "data.table" (default), "data.frame", or "sf".
Details
By default, this function draws a global sample of random locations. You can restrict it to a certain region via specifying xmin, xmax, ymin,
and ymax. The function draws from a uniform spatial distribution that assumes the planet to be a perfect sphere. The spherical assumption is common in GIS
functions, but deviates from Earth's exact shape.